#!/bin/sh
#
# autogen.sh
#
# This file is a part of Anubis.
# Copyright (C) 2001, 2002 The Anubis Team.
#
# Anubis is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Anubis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Anubis is released under the GPL with the additional exemption that
# compiling, linking, and/or using OpenSSL is allowed.
#

#
# Compile and install the server (/usr/sbin/anubis).
#

INFODIR=/usr/share/info
MANDIR=/usr/share/man

if test "$1" = "install"; then
 ./configure --prefix=/usr
 make clean
 make
 make install mandir=${MANDIR} infodir=${INFODIR}
 gzip -f --best ${MANDIR}/man1/anubis.1
 gzip -f --best ${INFODIR}/anubis.info*
 /sbin/install-info ${INFODIR}/anubis.info.gz ${INFODIR}/dir
 make distclean
fi

#
# Uninstall the server (/usr/sbin/anubis).
#

if test "$1" = "uninstall"; then
 ./configure --prefix=/usr
 make uninstall mandir=${MANDIR} infodir=${INFODIR}
 rm -f ${MANDIR}/man1/anubis.1.gz
 /sbin/install-info --delete ${INFODIR}/anubis.info.gz ${INFODIR}/dir
 rm -f ${INFODIR}/anubis.info*.gz
 make distclean
fi

#
# Create a private key and a certificate.
#

if test "$1" = "keygen"; then
 PEM=anubis.pem
 rm -f ${PEM} ${PEM}.1 ${PEM}.2
 openssl req -newkey rsa:1024 -nodes -keyout ${PEM}.1 -x509 -days 365 -out ${PEM}.2
 cat ${PEM}.1 >${PEM}
 echo "" >>${PEM}
 cat ${PEM}.2 >>${PEM}
 rm -f ${PEM}.1 ${PEM}.2
 chmod u=rw,g=,o= ${PEM}
fi

#
# Generate a gzipped source archive.
#

if test "$1" = "dist"; then
 ./configure
 make dist
 make distclean
 mv -vf anubis-*.tar.gz ../
fi

#
# Generate a gzipped source archive
# and both: binary and source RPM.
#

if test "$1" = "rpm"; then
 unset LD_PRELOAD
 unset LD_LIBRARY_PATH
 ./configure
 if test `id -u` = "0"; then
  make rpm-root
 else
  if test -d "/var/tmp"; then
   make rpm-user
  else
   echo "The /var/tmp directory is needed."
  fi
 fi
 make distclean
 mv -vf anubis-*.tar.gz ../
 mv -vf anubis-*.rpm ../
fi

# EOF

