/*
   auth.c

   This file is a part of Anubis.
   Copyright (C) 2001, 2002 The Anubis Team.

   Anubis is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   Anubis is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Anubis is released under the GPL with the additional exemption that
   compiling, linking, and/or using OpenSSL is allowed.
*/

#include "headers.h"
#include "extern.h"

void auth_tunnel(void)
{
	info(NORMAL, "Welcome user %s !", user_client);
	open_rcfile(CLIENT);
	read_rcfile(CLIENT);
	read_rcfile_allsection();
	regex_position = get_position(BEGIN_REGEX);
	return;
}

/***********************
 IDENT protocol support
************************/

int auth_ident(struct sockaddr_in *addr, char *user, int size)
{
	struct servent *sp;
	struct sockaddr_in ident;
	char buf[LINEBUFFER + 1];
	int sd = 0;

	if ((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		anubis_error("IDENT: socket() failed: %s.", strerror(errno));
		return -1;
	}
	memcpy(&ident, addr, sizeof(ident));
	sp = getservbyname("auth", "tcp");
	if (sp)
		ident.sin_port = sp->s_port;
	else
		ident.sin_port = htons(113); /* default IDENT port number */

	if (connect(sd, (struct sockaddr *)&ident, sizeof(ident)) < 0) {
		anubis_error("IDENT: connect() failed: %s.", strerror(errno));
		close_socket(sd);
		return -1;
	}
	info(VERBOSE, "IDENT: connected to %s:%u",
	inet_ntoa(ident.sin_addr), ntohs(ident.sin_port));

	#ifdef HAVE_SNPRINTF
	snprintf(buf, LINEBUFFER,
	#else
	sprintf(buf,
	#endif /* HAVE_SNPRINTF */
	"%u , %u"CRLF, ntohs(addr->sin_port), t_portnumber);

	if (send(sd, buf, strlen(buf), 0) == -1) {
		anubis_error("IDENT: send() failed: %s.", strerror(errno));
		close_socket(sd);
		return -1;
	}
	if (recvline(CLIENT, (void *)sd, buf, LINEBUFFER) == -1) {
		anubis_error("IDENT: recvline() failed: %s.", strerror(errno));
		close_socket(sd);
		return -1;
	}
	close_socket(sd);
	memset(user, 0, size);

	if (sscanf(buf, "%*[^:]: USERID :%*[^:]:%s", user) != 1) {
		info(VERBOSE, "IDENT: incorrect data.");
		return -1;
	}

	/******************************
         IDENTD DES decryption support
	*******************************/

	if (strstr(user, "[") && strstr(user, "]")) {
		int rs = 0;
		info(VERBOSE, "IDENT: data probably encrypted with DES...");
		external_program(&rs, IDECRYPT_PATH, user, buf, LINEBUFFER);
		if (rs == -1)
			return -1;

		if (sscanf(buf, "%*[^ ] %*[^ ] %*[^ ] %*[^ ] %*[^ ] %s", user) != 1) {
			info(VERBOSE, "IDENT: incorrect data (DES deciphered).");
			return -1;
		}
		else { /* UID deciphered */
			struct passwd *pwd;
			int uid = atoi(user);
			pwd = getpwuid(uid);
			if (pwd != 0)
				strncpy(user, (char *)pwd->pw_name, size - 1);
			else
				return -1;
		}
	}
	info(VERBOSE, "IDENT: resolved remote user to %s.", user);
	return 0;
}

/* EOF */

