/*
   errs.c

   This file is a part of Anubis.
   Copyright (C) 2001, 2002 The Anubis Team.

   Anubis is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   Anubis is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Anubis is released under the GPL with the additional exemption that
   compiling, linking, and/or using OpenSSL is allowed.
*/

#include "headers.h"
#include "extern.h"

void anubis_error(char *format, ...)
{
	va_list arglist;
	char txt[LINEBUFFER + 1];

	va_start(arglist, format);
	#ifdef HAVE_VSNPRINTF
	vsnprintf(txt, LINEBUFFER,
	#else
	vsprintf(txt,
	#endif /* HAVE_VSNPRINTF */
	format, arglist);
	va_end(arglist);

	topt |= T_ERROR;

	if (output != SILENT) {
		#ifdef HAVE_SYSLOG
		if ((topt & T_DAEMON) && !(topt & T_FOREGROUND)) {
			syslog(LOG_ERR | LOG_MAIL, txt);
			filelog(ERRORS, txt);
		}
		else
		#endif /* HAVE_SYSLOG */
			if (topt & T_FOREGROUND)
				mprintf(">>[%d] %s", (int)getpid(), txt);
			else
				mprintf(">>%s", txt);
	}
	errno = 0;
	if (!(topt & T_DAEMON) && !(topt & T_FOREGROUND))
		quit(EXIT_FAILURE);
	return;
}

void socket_error(void)
{
	anubis_error("Couldn't write on socket! %s.", strerror(errno));
	return;
}

void socks_error(char *txt)
{
	anubis_error("SOCKS proxy: %s", txt);
	return;
}

void hostname_error(char *host)
{
	if (h_errno == 0)
		return;

	if (h_errno == HOST_NOT_FOUND)
		anubis_error("Unknown host %s.", host);
	else if (h_errno == NO_ADDRESS)
		anubis_error("Host name is valid but does not have an IP address.");
	else if (h_errno == NO_RECOVERY)
		anubis_error("Unrecoverable name server error occured.");
	else if (h_errno == TRY_AGAIN)
		anubis_error("A temporary name server error occured. Try again later.");
	else
		anubis_error("Unknown DNS error %d.", h_errno);

	h_errno = 0;
	return;
}

/* EOF */

