/*
   headers.h

   This file is a part of Anubis.
   Copyright (C) 2001, 2002 The Anubis Team.

   Anubis is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   Anubis is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Anubis is released under the GPL with the additional exemption that
   compiling, linking, and/or using OpenSSL is allowed.
*/

#ifdef HAVE_CONFIG_H

 #include <config.h>

 #if defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
  #if defined(HAVE_OPENSSL_SSL_H) && !defined(NOSSL)
   #define HAVE_SSL
  #endif /* HAVE_OPENSSL_SSL_H and not NOSSL */
 #endif /* HAVE_LIBSSL and HAVE_LIBCRYPTO */
 #if defined(HAVE_LIBGPGME) && defined(HAVE_GPGME_H) && !defined(NOGPG)
  #define HAVE_GPG
 #endif /* HAVE_LIBGPGME and HAVE_GPGME_H and not NOGPG */
 #if defined(HAVE_LIBPCRE)
  #if defined(HAVE_PCRE_H) || defined(HAVE_PCRE_PCRE_H)
   #define HAVE_PCRE
  #endif /* HAVE_PCRE_H or HAVE_PCRE_PCRE_H */
 #endif /* HAVE_LIBPCRE */
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
  #define HAVE_REGEX
 #endif /* HAVE_REGEX_H and HAVE_REGCOMP */
 #if defined(HAVE_LIBPAM) && defined (HAVE_LIBPAM_MISC)
  #if defined(HAVE_SECURITY_PAM_APPL_H) && defined(HAVE_SECURITY_PAM_MISC_H)
   #define HAVE_PAM
  #endif /* HAVE_SECURITY_PAM_APPL_H and HAVE_SECURITY_PAM_MISC_H */
 #endif /* HAVE_LIBPAM and HAVE_LIBPAM_MISC */
 #if defined(HAVE_LIBWRAP) && defined(HAVE_LIBNSL) && defined(HAVE_TCPD_H)
  #define USE_LIBWRAP
 #endif /* HAVE_LIBWRAP and HAVE_LIBNSL and HAVE_TCPD_H */

#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#ifdef STDC_HEADERS
#include <stdlib.h>
#include <stdarg.h>
#endif /* STDC_HEADERS */
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif /* HAVE_MEMORY_H */
#include <signal.h>
#include <errno.h>
#include <ctype.h>
#include <fcntl.h>
#include <time.h>
#include <pwd.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#include <sys/file.h>
#include <sys/fcntl.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

#if defined(HAVE_GETRLIMIT) && defined(HAVE_SETRLIMIT)
 #include <sys/resource.h>
#endif /* HAVE_GETRLIMIT and HAVE_SETRLIMIT */
#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
#endif /* HAVE_ARPA_INET_H */
#if defined(HAVE_SYSLOG) && defined(HAVE_SYSLOG_H)
 #include <syslog.h>
#endif /* HAVE_SYSLOG and HAVE_SYSLOG_H */
#ifdef HAVE_PCRE
 #ifdef HAVE_PCRE_H
  #include <pcre.h>
 #else
  #ifdef HAVE_PCRE_PCRE_H
   #include <pcre/pcre.h>
  #endif /* HAVE_PCRE_PCRE_H */
 #endif /* HAVE_PCRE_H */
 #undef HAVE_REGEX
#else
 #ifdef HAVE_REGEX
  #include <regex.h>
 #else
  #error regex.h is missing! Try with './configure --with-pcre'.
 #endif /* HAVE_REGEX */
#endif /* HAVE_PCRE */

#ifdef HAVE_SSL
 #include <openssl/crypto.h>
 #include <openssl/x509.h>
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
 #include <openssl/md5.h>
#endif /* HAVE_SSL */

#ifdef HAVE_PAM
 #include <security/pam_appl.h>
 #include <security/pam_misc.h>
#endif /* HAVE_PAM */

#include "mem.h" /* xfree() */

#ifndef INADDR_NONE
 #define INADDR_NONE (unsigned long)0xffffffff
#endif /* not INADDR_NONE */
#ifndef INADDR_ANY
 #define INADDR_ANY (unsigned long)0x00000000
#endif /* not INADDR_ANY */
#ifndef INADDR_LOOPBACK
 #define INADDR_LOOPBACK (unsigned long)0x7f000001
#endif /* not INADDR_LOOPBACK */

#ifndef MAXPATHLEN
 #ifdef PATH_MAX
  #define MAXPATHLEN PATH_MAX
 #else
  #define MAXPATHLEN 1024
 #endif /* PATH_MAX */
#endif /* not MAXPATHLEN */
#ifndef NAME_MAX
 #define NAME_MAX 255
#endif /* not NAME_MAX */

#define MAXCLIENTS 50
#define LINEBUFFER 512
#define DATABUFFER 4096
#define DEFAULT_GLOBAL_RCFILE "/etc/anubisrc"
#define DEFAULT_LOCAL_RCFILE ".anubisrc"
#define DEFAULT_SSL_PEM "anubis.pem"
#define DEFAULT_SIGFILE ".signature"
#define BEGIN_DCM "@@"
#define BEGIN_CONTROL "---BEGIN CONTROL---"
#define BEGIN_REGEX "---BEGIN REGEX---"
#define BEGIN_ALL "---BEGIN ALL---"
#define BEGIN_EXTERNAL "---BEGIN EXTERNAL---"
#define END_CONFIG "---END---"
#define LF "\n"
#define CRLF "\r\n"

/* REGEX action methods */
#define COMMAND 0
#define HEADER  1

/* Tunnel methods */
#define CLIENT 0
#define SERVER 1

/* run control file access */
/*      CLIENT     0 (defined above) */
#define SUPERVISOR 1
#define INIT       2

/* output modes */
#define SILENT  0
#define NORMAL  1
#define VERBOSE 2
#define DEBUG   3

/* logging levels */
#define NONE   0
#define ERRORS 1
#define ALL    2

/* bit values for topt */
#define T_ERROR             0x00000001
#define T_WG1               0x00000002
#define T_WG2               0x00000004
#define T_SOCKS             0x00000008
#define T_SOCKSV4           0x00000010
#define T_SOCKSUSERNAME     0x00000020
#define T_SOCKSPASSWORD     0x00000040
#define T_FOREGROUND_INIT   0x00000080
#define T_FOREGROUND        0x00000100
#define T_DAEMON            0x00000200
#define T_STDINOUT          0x00000400
#define T_SSL               0x00000800
#define T_SSL_CLIENT        0x00001000
#define T_SSL_SERVER        0x00002000
#define T_SSL_FINISHED      0x00004000
#define T_SSL_ONEWAY        0x00008000
#define T_SSL_CKCLIENT      0x00010000
#define T_NAMES             0x00020000
#define T_LOCAL_MTA         0x00040000
#define T_ALLOW_LOCAL_MTA   0x00080000
#define T_RCEXECARGS        0x00100000
#define T_ALLOW_EXTERNAL    0x00200000
#define T_SUPERCLIENT       0x00400000
#define T_USER_NOTPRIVIL    0x00800000
#define T_STARTTLS          0x01000000
#define T_BOUNDARY          0x02000000
#define T_NORC              0x04000000
#define T_ALTRC             0x08000000

/* bit values for mopt */
#define M_GPG_ENCRYPT       0x00000001
#define M_GPG_SIGN          0x00000002
#define M_GPG_PASSPHRASE    0x00000004
#define M_RM                0x00000008
#define M_RMRRT             0x00000010
#define M_RMPOST            0x00000020
#define M_RMHEADER          0x00000040
#define M_RMLT              0x00000080
#define M_RMRLT             0x00000100
#define M_RMGPG             0x00000200
#define M_BODYATTACH        0x00000400
#define M_BODYCLEARATTACH   0x00000800
#define M_SIGNATURE         0x00001000
#define M_ROT13S            0x00002000
#define M_ROT13B            0x00004000
#define M_EXTBODYPROC       0x00008000

#define safe_strcpy(s, ct) \
 (s[sizeof(s) - 1] = '\0', strncpy((char *)s, (char *)ct, sizeof(s) - 1))

/* mem.c */
void *xmalloc(int);
void *xrealloc(void *, int);
char *allocbuf(char *, int);
#ifndef HAVE_STRDUP
 char *strdup(const char *);
#endif /* not HAVE_STRDUP */

/* env.c */
void get_options(int, char *[]);
void get_homedir(char *, char *, int);
void anubis_getlogin(char *, int);
void anubis_changeowner(char *);
int  check_superuser(void);
int  check_username(char *);
int  check_filemode(char *);
int  check_filename(char *);

/* errs.c */
void anubis_error(char *, ...);
void socket_error(void);
void socks_error(char *);
void hostname_error(char *);

/* log.c */
void mprintf(char *format, ...);
void info(int, char *, ...);
void filelog(int, char *);

/* net.c */
int  make_remote_connection(char *, unsigned int);
int  bind_and_listen(char *, unsigned int);
void swrite(int, void *, char *);
int  recvline(int, void *, void *, int);
void get_response_smtp(int, void *, char *, int);
void close_socket(int);

/* proxy.c */
void check_all_proxies(char *, unsigned int *);
int  check_wingates(int, char *, unsigned int);
int  check_socks_proxy(int, char *, unsigned int);

/* daemon.c */
void daemonize(void);
void loop(int);
void stdinout(void);

/* tunnel.c */
void smtp_session(void *, void *);

/* exec.c */
char **gen_execargs(const char *);
void free_execargs(char **);
int  make_local_connection(char *, char **);
char *external_program(int *, char *, char *, char *, int);

/* auth.c */
void auth_tunnel(void);
int  auth_ident(struct sockaddr_in *, char *, int);

/* misc.c */
void parse_hostname_port(char *, char *, unsigned int *);
void parse_hostname_host(char *, char *, unsigned int *);
void parse_extlist(int *, char *, char *, char *, int);
void remheaderline(char *);
void remline(char *, char *);
void remcrlf(char *);
void getoneline(char *, char *, int);
void change_to_lower(char *);
void check_rot13(void);

/* file.c */
void check_all_files(char *);

/* rc.c */
void open_rcfile(int);
void read_rcfile(int);
void read_rcfile_allsection(void);
void close_rcfile(void);
char *parse_line_option(char *);
int  regex_match(char *, char *);
int  read_regex_block(int, char *, int);
int  read_action_block(char *);
unsigned long get_position(char *);

/* help.c */
void print_version(void);
void print_usage(void);

/* quit.c */
void sig_exit(int);
void sig_timeout(int);
void free_mem(void);
void quit(int);

/* ssl.c */
#ifdef HAVE_SSL
 void init_ssl_libs(void);
 SSL_CTX *init_ssl_client(void);
 SSL_CTX *init_ssl_server(void);
 SSL *start_ssl_client(int, SSL_CTX *);
 SSL *start_ssl_server(int, SSL_CTX *);
 void end_ssl(int, SSL *, SSL_CTX *);
#endif /* HAVE_SSL */

/* gpg.c */
#ifdef HAVE_GPG
 void check_gpg(void);
#endif /* HAVE_GPG */

/* EOF */

