/*
   Anubis v3.4.6 - an outgoing mail processor and the SMTP tunnel.
   Copyright (C) 2001, 2002 The Anubis Team.

   Anubis is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   Anubis is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Anubis is released under the GPL with the additional exemption that
   compiling, linking, and/or using OpenSSL is allowed.
*/

#include "headers.h"
#include "main.h"

int main(int argc, char *argv[])
{
	#if !defined(GDEBUG) && defined(HAVE_GETRLIMIT) && defined(HAVE_SETRLIMIT)
	struct rlimit corelimit;
	if (getrlimit(RLIMIT_CORE, &corelimit) == 0) {
		corelimit.rlim_cur = 0;
		setrlimit(RLIMIT_CORE, &corelimit);
	}
	#endif /* not GDEBUG and HAVE_GETRLIMIT && HAVE_SETRLIMIT */

	/*
	   Signal handling.
	*/

	signal(SIGILL,  sig_exit);
	signal(SIGINT,  sig_exit);
	signal(SIGTERM, sig_exit);
	signal(SIGHUP,  sig_exit);
	signal(SIGQUIT, sig_exit);
	signal(SIGPIPE, SIG_IGN);
	signal(SIGALRM, sig_timeout);

	/*
	   Process the command line options.
	*/

	get_options(argc, argv);
	anubis_getlogin(user_supervisor, sizeof(user_supervisor));

	/*
	   Read the run control file (SUPERVISOR).
	*/

	if (!(topt & T_NORC)) {
		open_rcfile(SUPERVISOR);
		read_rcfile(INIT);
		close_rcfile(); /* SUPERVISOR */
	}

	/*
	   DEBUG
	*/

	#ifdef GDEBUG
	info(DEBUG, "UID:%d, GID:%d, EUID:%d, EGID:%d", (int)getuid(),
	(int)getgid(), (int)geteuid(), (int)getegid());
	#endif /* GDEBUG */

	/*
	   Initialize the OpenSSL library and the PRNG.
	*/

	#ifdef HAVE_SSL
	init_ssl_libs();
	#endif /* HAVE_SSL */

	/*
	   Enter the main core...
	*/

	dcmlen = strlen(BEGIN_DCM);
	if (topt & T_STDINOUT) { /* stdin/stdout */
		if (output != DEBUG)
			output = SILENT;
		topt &= ~T_SSL;
		topt |= T_FOREGROUND;
		stdinout();
	}
	else { /* daemon */
		int sd_bind = bind_and_listen(t_hostname, t_portnumber);
		if (topt & T_FOREGROUND_INIT)
			topt |= T_FOREGROUND;
		else
			daemonize();
		loop(sd_bind);
	}
	return 0;
}

/* EOF */

