/*
   misc.c

   This file is a part of Anubis.
   Copyright (C) 2001, 2002 The Anubis Team.

   Anubis is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   Anubis is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Anubis is released under the GPL with the additional exemption that
   compiling, linking, and/or using OpenSSL is allowed.
*/

#include "headers.h"
#include "extern.h"

/****************************
 An extra port number parser
*****************************/

void parse_hostname_port(char *opt, char *host, unsigned int *port)
{
	struct servent *p;
	char opt_tmp[256];
	char *port_tmp = 0;
	safe_strcpy(opt_tmp, opt);

	if ((port_tmp = strrchr(opt_tmp, ':'))) {
		*port_tmp++ = '\0';
		if ((p = getservbyname(port_tmp, "tcp")))
			*port = ntohs(p->s_port);
		else
			*port = (unsigned int)atoi(port_tmp);
	}
	strncpy(host, opt_tmp, sizeof(hostname) - 1);
	return;
}

/**************************
 An extra host name parser
***************************/

void parse_hostname_host(char *opt, char *host, unsigned int *port)
{
	struct servent *p;
	char opt_tmp[256];
	char *port_tmp = 0;
	safe_strcpy(opt_tmp, opt);

	if ((port_tmp = strrchr(opt_tmp, ':'))) {
		*port_tmp++ = '\0';
		if ((p = getservbyname(port_tmp, "tcp")))
			*port = ntohs(p->s_port);
		else
			*port = (unsigned int)atoi(port_tmp);
		strncpy(host, opt_tmp, sizeof(t_hostname) - 1);
	}
	else { /* port number available only */
		if ((p = getservbyname(opt, "tcp")))
			*port = ntohs(p->s_port);
		else
			*port = (unsigned int)atoi(opt);
	}
	return;
}

/*******************
 The extlist parser
********************/

void parse_extlist(int *cs, char *extuser, char *extaddr, char *dst, int size)
{
	char *p = 0;
	char *ptr1 = 0;
	char *ptr2 = 0;
	char a1[65];
	char a2[65];
	char external[65];
	char local[65];
	char user[65];
	char address[65];
	char oneline[LINEBUFFER + 1];
	int cu = 0; /* check a user name */
	unsigned long inaddr;
	struct sockaddr_in addr;

	memset(&addr, 0, sizeof(addr));
	*cs = 0; /* failed by default */

	if (extlist == 0)
		return;
	if (strlen(extlist) == 0)
		return;

	safe_strcpy(a1, extaddr);
	do
	{
		getoneline(extlist, oneline, LINEBUFFER);
		remcrlf(oneline);
		remline(extlist, oneline);

		if (regex_match("^[^ ]+[ \t]*=[ \t]*[^ ]+[ \t]+[^ ]+[ \t]*=[ \t]*[^ ]+",
			oneline) == 0) {
			info(VERBOSE, "EXTERNAL: incorrect syntax.");
			continue; /* failed */
		}
		ptr1 = parse_line_option(oneline);
		ptr2 = parse_line_option(ptr1);
		p = strchr(ptr1, ' ');
		*p = '\0';
		safe_strcpy(external, ptr1);
		safe_strcpy(local, ptr2);

		if (strstr(external, "@")) {
			if (extuser == 0)
				continue; /* failed */
			safe_strcpy(user, external);
			p = strchr(user, '@');
			*p++ = '\0';
			safe_strcpy(address, p);
			cu = 1;
		}
		else
			safe_strcpy(address, external);

		inaddr = inet_addr(address);
		if (inaddr != INADDR_NONE)
			memcpy(&addr.sin_addr, &inaddr, sizeof(inaddr));
		else {
			struct hostent *hp = 0;
			hp = gethostbyname(address);
			if (hp == 0) {
				hostname_error(address);
				continue; /* failed */
			}
			else {
				if (hp->h_length != 4 && hp->h_length != 8) {
					anubis_error("Illegal address length received for host %s", address);
					continue; /* failed */
				}
				else {
					memcpy((char *)&addr.sin_addr.s_addr, hp->h_addr,
					hp->h_length);
				}
			}
		}

		safe_strcpy(a2, inet_ntoa(addr.sin_addr));
		if (cu) {
			if (strcmp(extuser, user) == 0)
				if (strcmp(a1, a2) == 0) {
					*cs = 1; /* success */
					break;
				}
		}
		else {
			if (strcmp(a1, a2) == 0) {
				*cs = 1; /* success */
				break;
			}
		}
	} while (strlen(extlist) != 0);
	xfree(extlist);

	if (*cs) { /* success */
		if (check_username(local)) {
			info(NORMAL, "%s accepted as %s@localhost.", external, local);
			memset(dst, 0, size);
			strncpy(dst, local, size - 1);
		}
		else
			*cs = 0; /* failed */
	}
	return;
}

/**********************
 Remove a single line
 from a message header
***********************/

void remheaderline(char *line)
{
	int size;
	if (line == 0)
		return;

	size = strlen(line) + 2;
	if (remlist)
		size += strlen(remlist);
	remlist = (char *)xrealloc((char *)remlist, size);
	strncat(remlist, line, strlen(line));
	strncat(remlist, LF, 1);
	return;
}

/*********************
 Remove a single line
**********************/

void remline(char *s, char *line)
{
	char *pos1 = 0;
	char *pos2 = 0;
	int len;

	if (s == 0 || line == 0)
		return;
	pos1 = strstr(s, line);
	if (pos1 == 0)
		return;

	pos2 = pos1;
	do {
		pos2++;
	} while (*pos2 != '\n');
	pos2++;
	len = strlen(pos2);
	pos2 = (char *)memmove(pos1, pos2, len);
	pos2[len] = '\0';
	return;
}

void remcrlf(char *s)
{
	int len;

	if (s == 0)
		return;
	len = strlen(s);

	if (len >= 2 && s[len - 2] == '\r' && s[len - 1] == '\n') { /* CRLF */
		s[len - 2] = '\0';
		s[len - 1] = '\0';
	}
	else if (len >= 2 && s[len - 2] != '\r' && s[len - 1] == '\n') /* LF */
		s[len - 1] = '\0';
	else if (len >= 1 && s[len - 1] == '\r') /* CR */
		s[len - 1] = '\0';
	else if (len >= 1 && s[len - 1] == '\n') /* LF */
		s[len - 1] = '\0';
	return;
}

void getoneline(char *s, char *line, int maxsize)
{
	char *p = 0;
	int len;

	if (s == 0)
		return;
	p = strchr(s, '\n');
	if (p == 0)
		return;
	p++;
	len = strlen(s) - strlen(p);
	if (len > maxsize)
		len = maxsize;

	memset(line, 0, maxsize);
	strncpy(line, s, len);
	return;
}

/***************************
 change to lower characters
****************************/

void change_to_lower(char *s)
{
	int c;
	int len;

	if (s == 0)
		return;
	len = strlen(s);

	for (c = len - 1; c >= 0; c--)
		s[c] = tolower((unsigned char)s[c]);
	return;
}

/****************
 ROT-13 encoding
*****************/

void check_rot13(void)
{
	if (msgbody && (mopt & M_ROT13B)) {
		int c;
		char *txt = msgbody;
		for (c = strlen(txt) - 1; c >= 0; c--)
		{
			txt[c] = (islower((unsigned char)txt[c])
			? 'a'+ (txt[c] - 'a' + 13)%26 : isupper((unsigned char)txt[c])
			? 'A' + (txt[c] - 'A' + 13)%26 : txt[c]);
		}
	}
	return;
}

/* EOF */

