/*
   quit.c

   This file is a part of Anubis.
   Copyright (C) 2001, 2002 The Anubis Team.

   Anubis is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   Anubis is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Anubis is released under the GPL with the additional exemption that
   compiling, linking, and/or using OpenSSL is allowed.
*/

#include "headers.h"
#include "extern.h"

void sig_exit(int code)
{
	info(NORMAL, "Signal Caught. Exiting Cleanly...");
	quit(code);
}

void sig_timeout(int code)
{
	info(NORMAL, "Timeout! Exiting...");
	quit(code);
}

void free_mem(void)
{
	xfree(msgbody);
	xfree(body_attach);
	xfree(boundary);
	xfree(rm_rrt);
	xfree(rm_post);
	xfree(rm_header);
	xfree(rm_gpg_file);

	#ifdef HAVE_SSL
	xfree(ssl_cert);
	xfree(ssl_key);
	#endif /* HAVE_SSL */

	#ifdef HAVE_GPG
	xfree(gpg_keys);
	xfree(gpg_rm_key);
	if (gpg_passphrase) {
		memset(gpg_passphrase, 0, strlen(gpg_passphrase));
		xfree(gpg_passphrase);
	}
	#endif /* HAVE_GPG */

	xfree(extlist);
	xfree(logfile);
	xfree(execpath);
	if (topt & T_RCEXECARGS) {
		free_execargs(execargs);
		topt &= ~T_RCEXECARGS;
	}
	return;
}

void quit(int code)
{
	#ifdef HAVE_SSL
	end_ssl(CLIENT, ssl_client, ctx_client);
	end_ssl(SERVER, ssl_server, ctx_server);
	#endif /* HAVE_SSL */

	#ifdef HAVE_SYSLOG
	if ((topt & T_DAEMON) && !(topt & T_FOREGROUND))
		closelog();
	#endif /* HAVE_SYSLOG */

	close_rcfile();
	free_mem();
	exit(code);
}

/* EOF */

